--
--
-- TOP_SPIM module
--
-- VHDL synthesis and simulation model of MIPS machine
-- as described in chapter 5 of Patterson and Hennessey
-- NOTE: Data paths limited to 8 bits to speed synthesis and
-- simulation.  Registers limited to 8 bits and $R0..$R7
-- Program and Data memory limited to locations 0..7

library synth;
use synth.stdsynth.ALL;

entity TOP_SPIM is

port(reset,phi1,phi2: in vlbit; PC: out vlbit_1d(7 downto 0));

end TOP_SPIM;



architecture structure of TOP_SPIM is

   component Ifetch

   port(Instruction : out vlbit_1d(31 downto 0);
		PCadd : out  vlbit_1d(7 downto 0);
	Addresult : in vlbit_1d(7 downto 0);
	Branch : in vlbit;
		phi1,phi2,reset : in vlbit;
	Zero : in vlbit;
	PCout : out vlbit_1d(7 downto 0));

   end component;

   component Idecode

   port(rr1d_bus : out vlbit_1d(7 downto 0);
	rr1a_bus : in vlbit_1d(4 downto 0);
	rr2d_bus : out vlbit_1d(7 downto 0);
	rr2a_bus : in vlbit_1d(4 downto 0);
	wrd_bus : in vlbit_1d(7 downto 0);
	wra_bus : in vlbit_1d(4 downto 0);
	wra2_bus : in vlbit_1d(4 downto 0);
	RegWrite : in vlbit;
	RegDst : in vlbit;
	Ivalue : in vlbit_1d(15 downto 0);
	Extend : out vlbit_1d(7 downto 0);
		phi1,phi2,reset: in vlbit);

	end component;



   component control


   port( Op : in vlbit_1d(5 downto 0);
		RegDst : out vlbit;
		ALUSrc : out vlbit;
		MemtoReg : out vlbit;
		RegWrite : out vlbit;
		MemRead : out vlbit;
		MemWrite : out vlbit;
		Branch : out vlbit;
		ALUop0 : out vlbit;
		ALUop1 : out vlbit;
		phi1,phi2: in vlbit);
	
	 end component;

   component  Execute


   port(Readdata1 : in vlbit_1d(7 downto 0);
		Readdata2 : in vlbit_1d(7 downto 0);
		Extend : in vlbit_1d(7 downto 0);
		Func : in vlbit_1d(5 downto 0);
		ALUOp0 : in vlbit;
		ALUOp1 : in vlbit; 
		ALUSrc : in vlbit;
		Zero : out vlbit;
		ALUResult : out vlbit_1d(7 downto 0);
		ADDResult : out vlbit_1d(7 downto 0);
		PCadd : in vlbit_1d(7 downto 0);
		phi1,phi2: in vlbit);

	end component;

   component dmemory

   port(rd_bus : out vlbit_1d(7 downto 0);
	ra_bus : in vlbit_1d(7 downto 0);
	wd_bus : in vlbit_1d(7 downto 0);
	wa_bus : in vlbit_1d(7 downto 0);
	MemRead, Memwrite, MemtoReg : in vlbit;
	phi1,phi2,reset: in vlbit);

   end component;

   signal Instruction :  vlbit_1d(31 downto 0);
   signal PCadd :  vlbit_1d(7 downto 0);
   signal rr1d_bus : vlbit_1d(7 downto 0);
   signal rr2d_bus : vlbit_1d(7 downto 0);
   signal Extend : vlbit_1d(7 downto 0);
   signal Addresult : vlbit_1d(7 downto 0);
   signal ALUresult : vlbit_1d(7 downto 0);
   signal Branch : vlbit;
   signal Zero : vlbit;
   signal wrd_bus : vlbit_1d(7 downto 0);
   signal RegWrite : vlbit;
   signal RegDst :  vlbit;
   signal ALUSrc :  vlbit;
   signal MemtoReg :  vlbit;
   signal MemRead :  vlbit;
   signal MemWrite :  vlbit;
   signal ALUop :  vlbit_1d(1 downto 0);
 

begin


   IFE : Ifetch

   port map (Instruction => Instruction,
		PCadd => PCadd,
	Addresult => Addresult,
	Branch => Branch,
		phi1 => phi1, phi2 => phi2, reset => reset,
	Zero => Zero,
	PCout => PC);

   ID : Idecode

   port map (rr1d_bus => rr1d_bus,
	rr1a_bus => Instruction(25 downto 21),
	rr2d_bus => rr2d_bus,
	rr2a_bus => Instruction(20 downto 16),
	wrd_bus => wrd_bus,
	wra_bus =>  Instruction(15 downto 11),
	wra2_bus =>  Instruction(20 downto 16),
	RegWrite => RegWrite,
	RegDst => RegDst,
	Ivalue => Instruction(15 downto 0),
	Extend => Extend,
		phi1 => phi1, phi2 => phi2, reset => reset);

   CTL:   control

   port map ( Op => Instruction(31 downto 26),
		RegDst => RegDst,
		ALUSrc => ALUSrc,
		MemtoReg => MemtoReg,
		RegWrite => RegWrite,
		MemRead => MemRead,
		MemWrite => MemWrite,
		Branch => Branch,
		ALUop0 => ALUop(0),
		ALUop1 => ALUop(1),
		phi1 => phi1, phi2 => phi2);

   EXE:  Execute


   port map (Readdata1 => rr1d_bus,
		Readdata2 => rr2d_bus,
		Extend => Extend,
		Func => Instruction(5 downto 0),
		ALUOp0 => ALUop(0),
		ALUOp1 => ALUop(1), 
		ALUSrc => ALUSrc,
		Zero => Zero,
		ALUResult => ALUResult,
		ADDResult => ADDResult,
		PCadd => PCadd,
		phi1 => phi1, phi2 => phi2);

   MEM:  dmemory

   port map (rd_bus => wrd_bus,
	ra_bus => ALUResult,
	wd_bus => rr2d_bus,
	wa_bus => ALUResult,
	MemRead => MemRead, 
	Memwrite => MemWrite, 
	MemtoReg => MemtoReg,
		phi1 => phi1, phi2 => phi2, reset => reset);
	
end structure;

